// Generated by CoffeeScript 1.3.1
var Connection, DEFAULT_CANCEL_TIMEOUT, DEFAULT_CLIENT_REQUEST_TIMEOUT, DEFAULT_CONNECT_TIMEOUT, DEFAULT_PACKET_SIZE, DEFAULT_PORT, DEFAULT_TEXTSIZE, Debug, EventEmitter, KEEP_ALIVE_INITIAL_DELAY, Login7Payload, MessageIO, PreloginPayload, RpcRequestPayload, Socket, SqlBatchPayload, TYPE, TokenStreamParser, instanceLookup,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; },
  __slice = [].slice;

require('./buffertools');

Debug = require('./debug');

EventEmitter = require('events').EventEmitter;

instanceLookup = require('./instance-lookup').instanceLookup;

TYPE = require('./packet').TYPE;

PreloginPayload = require('./prelogin-payload');

Login7Payload = require('./login7-payload');

RpcRequestPayload = require('./rpcrequest-payload');

SqlBatchPayload = require('./sqlbatch-payload');

MessageIO = require('./message-io');

Socket = require('net').Socket;

TokenStreamParser = require('./token/token-stream-parser').Parser;

KEEP_ALIVE_INITIAL_DELAY = 30 * 1000;

DEFAULT_CONNECT_TIMEOUT = 15 * 1000;

DEFAULT_CLIENT_REQUEST_TIMEOUT = 15 * 1000;

DEFAULT_CANCEL_TIMEOUT = 5 * 1000;

DEFAULT_PACKET_SIZE = 4 * 1024;

DEFAULT_TEXTSIZE = '2147483647';

DEFAULT_PORT = 1433;

Connection = (function(_super) {

  __extends(Connection, _super);

  Connection.name = 'Connection';

  Connection.prototype.STATE = {
    CONNECTING: {
      name: 'Connecting',
      enter: function() {
        return this.initialiseConnection();
      },
      events: {
        socketError: function(error) {
          return this.transitionTo(this.STATE.FINAL);
        },
        connectTimeout: function() {
          return this.transitionTo(this.STATE.FINAL);
        },
        socketConnect: function() {
          this.sendPreLogin();
          return this.transitionTo(this.STATE.SENT_PRELOGIN);
        }
      }
    },
    SENT_PRELOGIN: {
      name: 'SentPrelogin',
      enter: function() {
        return this.emptyMessageBuffer();
      },
      events: {
        socketError: function(error) {
          return this.transitionTo(this.STATE.FINAL);
        },
        connectTimeout: function() {
          return this.transitionTo(this.STATE.FINAL);
        },
        packet: function(packet) {
          return this.addToMessageBuffer(packet);
        },
        message: function() {
          this.processPreLoginResponse();
          this.sendLogin7Packet();
          return this.transitionTo(this.STATE.SENT_LOGIN7_WITH_STANDARD_LOGIN);
        }
      }
    },
    SENT_LOGIN7_WITH_STANDARD_LOGIN: {
      name: 'SentLogin7WithStandardLogin',
      events: {
        socketError: function(error) {
          return this.transitionTo(this.STATE.FINAL);
        },
        connectTimeout: function() {
          return this.transitionTo(this.STATE.FINAL);
        },
        packet: function(packet) {
          return this.sendPacketToTokenStreamParser(packet);
        },
        loggedIn: function() {
          return this.transitionTo(this.STATE.LOGGED_IN_SENDING_INITIAL_SQL);
        },
        loginFailed: function() {
          return this.transitionTo(this.STATE.FINAL);
        },
        message: function() {
          return this.processLogin7Response();
        }
      }
    },
    LOGGED_IN_SENDING_INITIAL_SQL: {
      name: 'LoggedInSendingInitialSql',
      enter: function() {
        return this.sendInitialSql();
      },
      events: {
        packet: function(packet) {
          return this.sendPacketToTokenStreamParser(packet);
        },
        message: function(error) {
          this.transitionTo(this.STATE.LOGGED_IN);
          return this.processedInitialSql();
        }
      }
    },
    LOGGED_IN: {
      name: 'LoggedIn',
      events: {
        socketError: function(error) {
          return this.transitionTo(this.STATE.FINAL);
        }
      }
    },
    SENT_CLIENT_REQUEST: {
      name: 'SentClientRequest',
      events: {
        socketError: function(error) {
          return this.transitionTo(this.STATE.FINAL);
        },
        packet: function(packet) {
          return this.sendPacketToTokenStreamParser(packet);
        },
        message: function() {
          var sqlRequest;
          this.transitionTo(this.STATE.LOGGED_IN);
          sqlRequest = this.request;
          this.request = void 0;
          return sqlRequest.callback(sqlRequest.error, sqlRequest.rowCount);
        }
      }
    },
    FINAL: {
      name: 'Final',
      enter: function() {
        return this.cleanupConnection();
      },
      events: {
        loginFailed: function() {},
        connectTimeout: function() {}
      }
    }
  };

  function Connection(config) {
    this.config = config;
    this.socketClose = __bind(this.socketClose, this);

    this.socketConnect = __bind(this.socketConnect, this);

    this.socketError = __bind(this.socketError, this);

    this.connectTimeout = __bind(this.connectTimeout, this);

    this.defaultConfig();
    this.createDebug();
    this.createTokenStreamParser();
    this.transitionTo(this.STATE.CONNECTING);
  }

  Connection.prototype.close = function() {
    return this.transitionTo(this.STATE.FINAL);
  };

  Connection.prototype.initialiseConnection = function() {
    this.connect();
    return this.createConnectTimer();
  };

  Connection.prototype.cleanupConnection = function() {
    if (!this.closed) {
      this.clearConnectTimer();
      this.closeConnection();
      this.emit('end');
      return this.closed = true;
    }
  };

  Connection.prototype.defaultConfig = function() {
    var _base, _base1, _base2, _base3, _base4, _base5;
    (_base = this.config).options || (_base.options = {});
    (_base1 = this.config.options).textsize || (_base1.textsize = DEFAULT_TEXTSIZE);
    (_base2 = this.config.options).connectTimeout || (_base2.connectTimeout = DEFAULT_CONNECT_TIMEOUT);
    (_base3 = this.config.options).requestTimeout || (_base3.requestTimeout = DEFAULT_CLIENT_REQUEST_TIMEOUT);
    (_base4 = this.config.options).cancelTimeout || (_base4.cancelTimeout = DEFAULT_CANCEL_TIMEOUT);
    (_base5 = this.config.options).packetSize || (_base5.packetSize = DEFAULT_PACKET_SIZE);
    if (!this.config.options.port && !this.config.options.instanceName) {
      return this.config.options.port = DEFAULT_PORT;
    } else if (this.config.options.port && this.config.options.instanceName) {
      throw new Error("Port and instanceName are mutually exclusive, but " + config.options.port + " and " + config.options.instanceName + " provided");
    }
  };

  Connection.prototype.createDebug = function() {
    var _this = this;
    this.debug = new Debug(this.config.options.debug);
    return this.debug.on('debug', function(message) {
      return _this.emit('debug', message);
    });
  };

  Connection.prototype.createTokenStreamParser = function() {
    var _this = this;
    this.tokenStreamParser = new TokenStreamParser(this.debug);
    this.tokenStreamParser.on('infoMessage', function(token) {
      return _this.emit('infoMessage', token);
    });
    this.tokenStreamParser.on('errorMessage', function(token) {
      _this.emit('errorMessage', token);
      if (_this.request) {
        return _this.request.error = token.message;
      }
    });
    this.tokenStreamParser.on('databaseChange', function(token) {
      return _this.emit('databaseChange', token.newValue);
    });
    this.tokenStreamParser.on('languageChange', function(token) {
      return _this.emit('languageChange', token.newValue);
    });
    this.tokenStreamParser.on('charsetChange', function(token) {
      return _this.emit('charsetChange', token.newValue);
    });
    this.tokenStreamParser.on('loginack', function(token) {
      return _this.loggedIn = true;
    });
    this.tokenStreamParser.on('packetSizeChange', function(token) {
      return _this.messageIo.packetSize(token.newValue);
    });
    this.tokenStreamParser.on('columnMetadata', function(token) {
      if (_this.request) {
        return _this.request.emit('columnMetadata', token.columns);
      } else {
        throw new Error("Received 'columnMetadata' when no sqlRequest is in progress");
      }
    });
    this.tokenStreamParser.on('order', function(token) {
      if (_this.request) {
        return _this.request.emit('order', token.orderColumns);
      } else {
        throw new Error("Received 'order' when no sqlRequest is in progress");
      }
    });
    this.tokenStreamParser.on('row', function(token) {
      if (_this.request) {
        _this.request.rowCount++;
        return _this.request.emit('row', token.columns);
      } else {
        throw new Error("Received 'row' when no sqlRequest is in progress");
      }
    });
    this.tokenStreamParser.on('returnStatus', function(token) {
      if (_this.request) {
        return _this.procReturnStatusValue = token.value;
      }
    });
    this.tokenStreamParser.on('returnValue', function(token) {
      if (_this.request) {
        return _this.request.emit('returnValue', token.paramName, token.value, token.metadata);
      }
    });
    this.tokenStreamParser.on('doneProc', function(token) {
      if (_this.request) {
        _this.request.emit('doneProc', token.rowCount, token.more, _this.procReturnStatusValue);
        return _this.procReturnStatusValue = void 0;
      }
    });
    this.tokenStreamParser.on('doneInProc', function(token) {
      if (_this.request) {
        return _this.request.emit('doneInProc', token.rowCount, token.more);
      }
    });
    return this.tokenStreamParser.on('done', function(token) {
      if (_this.request) {
        return _this.request.emit('done', token.rowCount, token.more);
      }
    });
  };

  Connection.prototype.connect = function() {
    var _this = this;
    if (this.config.options.port) {
      return this.connectOnPort(this.config.options.port);
    } else {
      return instanceLookup(this.config.server, this.config.options.instanceName, function(err, port) {
        if (err) {
          throw new Error(err);
        } else {
          return _this.connectOnPort(port);
        }
      });
    }
  };

  Connection.prototype.connectOnPort = function(port) {
    var _this = this;
    this.socket = new Socket({});
    this.socket.setKeepAlive(true, KEEP_ALIVE_INITIAL_DELAY);
    this.socket.connect(port, this.config.server);
    this.socket.on('error', this.socketError);
    this.socket.on('connect', this.socketConnect);
    this.socket.on('close', this.socketClose);
    this.socket.on('end', this.socketClose);
    this.messageIo = new MessageIO(this.socket, this.config.options.packetSize, this.debug);
    this.messageIo.on('packet', function(packet) {
      return _this.dispatchEvent('packet', packet);
    });
    return this.messageIo.on('message', function() {
      return _this.dispatchEvent('message');
    });
  };

  Connection.prototype.closeConnection = function() {
    return this.socket.destroy();
  };

  Connection.prototype.createConnectTimer = function() {
    return this.connectTimer = setTimeout(this.connectTimeout, this.config.options.connectTimeout);
  };

  Connection.prototype.connectTimeout = function() {
    var message;
    message = "timeout : failed to connect to " + this.config.server + ":" + this.config.options.port + " in " + this.config.options.connectTimeout + "ms";
    this.debug.log(message);
    this.emit('connect', message);
    this.connectTimer = void 0;
    return this.dispatchEvent('connectTimeout');
  };

  Connection.prototype.clearConnectTimer = function() {
    if (this.connectTimer) {
      return clearTimeout(this.connectTimer);
    }
  };

  Connection.prototype.transitionTo = function(newState) {
    var _ref, _ref1;
    if ((_ref = this.state) != null ? _ref.exit : void 0) {
      this.state.exit.apply(this);
    }
    this.debug.log("State change: " + ((_ref1 = this.state) != null ? _ref1.name : void 0) + " -> " + newState.name);
    this.state = newState;
    if (this.state.enter) {
      return this.state.enter.apply(this);
    }
  };

  Connection.prototype.dispatchEvent = function() {
    var args, eventFunction, eventName, _ref;
    eventName = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    if ((_ref = this.state) != null ? _ref.events[eventName] : void 0) {
      return eventFunction = this.state.events[eventName].apply(this, args);
    } else {
      throw new Error("No event '" + eventName + "' in state '" + this.state.name + "'");
    }
  };

  Connection.prototype.socketError = function(error) {
    var message;
    message = "connection to " + this.config.server + ":" + this.config.options.port + " - failed " + error;
    this.debug.log(message);
    return this.dispatchEvent('socketError', error);
  };

  Connection.prototype.socketConnect = function() {
    this.debug.log("connected to " + this.config.server + ":" + this.config.options.port);
    return this.dispatchEvent('socketConnect');
  };

  Connection.prototype.socketClose = function() {
    this.debug.log("connection to " + this.config.server + ":" + this.config.options.port + " closed");
    return this.transitionTo(this.STATE.FINAL);
  };

  Connection.prototype.sendPreLogin = function() {
    var payload;
    payload = new PreloginPayload();
    this.messageIo.sendMessage(TYPE.PRELOGIN, payload.data);
    return this.debug.payload(function() {
      return payload.toString('  ');
    });
  };

  Connection.prototype.emptyMessageBuffer = function() {
    return this.messageBuffer = new Buffer(0);
  };

  Connection.prototype.addToMessageBuffer = function(packet) {
    return this.messageBuffer = this.messageBuffer.concat(packet.data());
  };

  Connection.prototype.processPreLoginResponse = function() {
    var preloginPayload;
    preloginPayload = new PreloginPayload(this.messageBuffer);
    return this.debug.payload(function() {
      return preloginPayload.toString('  ');
    });
  };

  Connection.prototype.sendLogin7Packet = function() {
    var loginData, payload;
    loginData = {
      userName: this.config.userName,
      password: this.config.password,
      database: this.config.options.database,
      packetSize: this.config.options.packetSize
    };
    payload = new Login7Payload(loginData);
    this.messageIo.sendMessage(TYPE.LOGIN7, payload.data);
    return this.debug.payload(function() {
      return payload.toString('  ');
    });
  };

  Connection.prototype.sendPacketToTokenStreamParser = function(packet) {
    return this.tokenStreamParser.addBuffer(packet.data());
  };

  Connection.prototype.sendInitialSql = function() {
    var payload;
    payload = new SqlBatchPayload('set textsize ' + this.config.options.textsize);
    return this.messageIo.sendMessage(TYPE.SQL_BATCH, payload.data);
  };

  Connection.prototype.processedInitialSql = function() {
    this.clearConnectTimer();
    return this.emit('connect');
  };

  Connection.prototype.processLogin7Response = function() {
    if (this.loggedIn) {
      return this.dispatchEvent('loggedIn');
    } else {
      this.emit('connect', 'Login failed; one or more errorMessage events should have been emitted');
      return this.dispatchEvent('loginFailed');
    }
  };

  Connection.prototype.execSqlBatch = function(request) {
    return this.makeRequest(request, TYPE.SQL_BATCH, new SqlBatchPayload(request.sqlTextOrProcedure));
  };

  Connection.prototype.execSql = function(request) {
    request.transformIntoExecuteSqlRpc();
    return this.makeRequest(request, TYPE.RPC_REQUEST, new RpcRequestPayload(request));
  };

  Connection.prototype.prepare = function(request) {
    request.transformIntoPrepareRpc();
    return this.makeRequest(request, TYPE.RPC_REQUEST, new RpcRequestPayload(request));
  };

  Connection.prototype.unprepare = function(request) {
    request.transformIntoUnprepareRpc();
    return this.makeRequest(request, TYPE.RPC_REQUEST, new RpcRequestPayload(request));
  };

  Connection.prototype.execute = function(request, parameters) {
    request.transformIntoExecuteRpc(parameters);
    return this.makeRequest(request, TYPE.RPC_REQUEST, new RpcRequestPayload(request));
  };

  Connection.prototype.callProcedure = function(request) {
    return this.makeRequest(request, TYPE.RPC_REQUEST, new RpcRequestPayload(request));
  };

  Connection.prototype.makeRequest = function(request, packetType, payload) {
    var message;
    if (this.state !== this.STATE.LOGGED_IN) {
      console.trace();
      message = "Invalid state; requests can only be made in the " + this.STATE.LOGGED_IN.name + " state, not the " + this.state.name + " state";
      this.debug.log(message);
      return request.callback(message);
    } else {
      this.request = request;
      this.request.rowCount = 0;
      this.messageIo.sendMessage(packetType, payload.data);
      this.debug.payload(function() {
        return payload.toString('  ');
      });
      return this.transitionTo(this.STATE.SENT_CLIENT_REQUEST);
    }
  };

  return Connection;

})(EventEmitter);

module.exports = Connection;
