// Generated by CoffeeScript 1.3.1
var EventEmitter, MessageIO, Packet, isPacketComplete, packetHeaderLength, packetLength,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

require('./buffertools');

EventEmitter = require('events').EventEmitter;

isPacketComplete = require('./packet').isPacketComplete;

packetLength = require('./packet').packetLength;

packetHeaderLength = require('./packet').HEADER_LENGTH;

Packet = require('./packet').Packet;

MessageIO = (function(_super) {

  __extends(MessageIO, _super);

  MessageIO.name = 'MessageIO';

  function MessageIO(socket, _packetSize, debug) {
    this.socket = socket;
    this._packetSize = _packetSize;
    this.debug = debug;
    this.sendPacket = __bind(this.sendPacket, this);

    this.eventData = __bind(this.eventData, this);

    this.socket.addListener('data', this.eventData);
    this.packetDataSize = this._packetSize - packetHeaderLength;
    this.packetBuffer = new Buffer(0);
    this.payloadBuffer = new Buffer(0);
  }

  MessageIO.prototype.eventData = function(data) {
    var length, packet, _results;
    this.packetBuffer = new Buffer(this.packetBuffer.concat(data));
    _results = [];
    while (isPacketComplete(this.packetBuffer)) {
      length = packetLength(this.packetBuffer);
      packet = new Packet(this.packetBuffer.slice(0, length));
      this.logPacket('Received', packet);
      this.emit('packet', packet);
      if (packet.isLast()) {
        this.emit('message');
      }
      _results.push(this.packetBuffer = new Buffer(this.packetBuffer.slice(length)));
    }
    return _results;
  };

  MessageIO.prototype.packetSize = function(packetSize) {
    if (arguments.length > 0) {
      this.debug.log("Packet size changed from " + this._packetSize + " to " + packetSize);
      this._packetSize = packetSize;
      this.packetDataSize = this._packetSize - packetHeaderLength;
    }
    return this._packetSize;
  };

  MessageIO.prototype.sendMessage = function(packetType, data) {
    var numberOfPackets, packet, packetNumber, packetPayload, payloadEnd, payloadStart, _i, _ref, _results;
    numberOfPackets = (Math.floor((data.length - 1) / this.packetDataSize)) + 1;
    _results = [];
    for (packetNumber = _i = 0, _ref = numberOfPackets - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; packetNumber = 0 <= _ref ? ++_i : --_i) {
      payloadStart = packetNumber * this.packetDataSize;
      if (packetNumber < numberOfPackets - 1) {
        payloadEnd = payloadStart + this.packetDataSize;
      } else {
        payloadEnd = data.length;
      }
      packetPayload = data.slice(payloadStart, payloadEnd);
      packet = new Packet(packetType);
      packet.last(packetNumber === numberOfPackets - 1);
      packet.packetId(packetNumber + 1);
      packet.addData(packetPayload);
      _results.push(this.sendPacket(packet));
    }
    return _results;
  };

  MessageIO.prototype.sendPacket = function(packet) {
    this.logPacket('Sent', packet);
    return this.socket.write(packet.buffer);
  };

  MessageIO.prototype.logPacket = function(direction, packet) {
    this.debug.packet(direction, packet);
    return this.debug.data(packet);
  };

  return MessageIO;

})(EventEmitter);

module.exports = MessageIO;
